#
# Server startup and configuration template. This file should be copied
# to conf/<servername>.<port>. It is used to start the server with most
# basic configuration.
# Users can modify copy of this file by adding server configuration
# options which may override built-in defaults.
#
# See the file "license.txt" for information on usage and
# redistribution of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#
# ----------------------------------------------------------------------
#

#
# Load and run the basic configuration procedure.
#
#  *** Do not modify the following line ***
#  *** It has to be the very first line ***
#

source [file dirname [ns_info config]]/../etc/srvconf.tcl

#
# Add additional config options below.
#
#
# Example: uncomment/adjust to relocate default TEMP directory elsewhere
#
# ns_section ns/parameters
# ns_param   tmpdir /some/new/place/for/temp/dir
#
#
# Example: uncomment to turn on extensive (developer) logging
#
# ns_section ns/parameters
# ns_param   logdev 1
#
#
# Example: uncomment to turn on SCSI command error logging
#
# ns_section ns/libarchdev
# ns_param   scsilog 1
#
#
# Example: uncomment to turn on extensive SCSI command logging
#
# ns_section ns/libarchdev
# ns_param   scsilog 2
#
#
# Example: uncomment to enable HTTP/HTTPS objectstore proxy support.
# Replace values for "host" and "port" parameters accordingly.
#
# ns_section ns/server/$server/module/objectstore/proxy
# ns_param   host localhost
# ns_param   port 3128

# EOF
