/* 
 *  Archiware P5 white label css file 
 */

.adminp5:before {
  content: "Product Label";
}
.t30welcome:before {
  content: "Welcome to Product Label";
}
.innerAdmin {
    position: absolute;
    background: url("/white-label/img/logo.png") no-repeat center;
    background-position: 20px 10px;
    background-size: 250px;
    width:620px;
	height:300px;
    margin-left: -60px;
    margin-top:-135px;
    left: 50%;
}
.adminp5 {
    font-size:3.4em;
	font-weight: bold;
    color: #5D646F;
}
.p5_256 {
    background: url("/white-label/img/logo.png") no-repeat center;
    background-position: 0px 0px;
    background-size: 160px;
    width:160px;
	height:160px;
}